/* Copyright (C) 2004 Hiroyuki OYAMA

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "module.h"
#include "module_list.h"
#include "httpd.h"

int mtb_validate_ping(request_rec *r, mtb_message_rec *msg)
{
    mtb_module_rec **cur;
    int rc;

    for (cur = mtb_root_module; *cur != NULL; cur++) {
        mtb_module_rec *m = *cur;
        if (m->validate_ping == NULL)
            continue;
        rc = m->validate_ping(r, msg);
        if (rc != OK && rc != DECLINED)
            return rc;
    }
    return OK;
}


int mtb_send_contents(request_rec *r, mtb_message_rec *msg)
{
    mtb_module_rec **cur;
    int rc;
    for (cur = mtb_root_module; *cur != NULL; cur++) {
        mtb_module_rec *m = *cur;
        if (m->send_contents == NULL)
            continue;
        rc = m->send_contents(r, msg);
        if (rc == DECLINED)
            continue;
        return rc;
    }
    return BAD_REQUEST;
}

int mtb_store_message(request_rec *r, mtb_message_rec *msg)
{
    mtb_module_rec **cur;
    int rc;

    for (cur = mtb_root_module; *cur != NULL; cur++) {
        mtb_module_rec *m = *cur;
        if (m->store_message == NULL)
            continue;
        rc = m->store_message(r, msg);
        if (rc == DECLINED)
            continue;
        return rc;
    }
    return SERVER_ERROR;
}

mtb_message_rec *mtb_load_message(request_rec *r, const char *id)
{
    mtb_message_rec *msg;
    mtb_module_rec **cur;

    for (cur = mtb_root_module; *cur != NULL; cur++) {
        mtb_module_rec *m = *cur;
        if (m->load_message == NULL)
            continue;
        msg = m->load_message(r, id);
        return msg;
    }
    return NULL;
}

