/* Copyright (C) 2004 Hiroyuki OYAMA

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "module.h"
#include "trackback.h"
#include "encoding.h"
#include "util.h"

static int send_html_contents(request_rec *r, mtb_message_rec *msg)
{
    const char *org_encoding, *encoding, *mode;
    char *body;
    table *param;
    struct tm *tms;
    unsigned char timestr[HUGE_STRING_LEN];
    iconv_t cd;

    param = (table *)ap_table_get(r->notes, REQUEST_PARAM_FIELD);
    mode = ap_table_get(param, "__mode");
    if (strcasecmp(mode, "html") != 0)
        return DECLINED;

    org_encoding = ap_table_get(param, "charset");
    if (org_encoding == NULL)
        org_encoding = "UTF-8";
    encoding = mtb_find_encoding(org_encoding);
    if ((cd = mtb_iconv_open(r->pool, "UTF-8", encoding)) == (iconv_t)(-1)) {
        /*
        ap_log_rerror(APLOG_MARK, APLOG_WARNING|APLOG_NOERRNO, r,
                      "cannot open iconv conversion descriptor: from=%s to=%s: %s",
                      "UTF-8", encoding, r->uri);
        */
    }


    r->content_type = "text/html";
    ap_send_http_header(r);

    ap_rprintf(r,
               "<html>\n"
               "  <head>\n"
               "    <meta http-equiv=\"Content-Type\" content=\"text/html; charset=%s\" />\n"
               "  </head>\n"
               "  <body>\n"
               "<!--\n"
               "<rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"\n"
               "         xmlns:trackback=\"http://madskills.com/public/xml/rss/module/trackback/\"\n"
               "         xmlns:dc=\"http://purl.org/dc/elements/1.1/\">\n"
               "<rdf:Description\n"
               "     trackback:ping=\"%s\"\n"
               "     />\n"
               "</rdf:RDF>\n"
               "-->\n"
               "    <div class=\"mtb_body\">\n"
               "      <div class=\"mtb_trackback_url\">%s</div>\n",
               org_encoding,
               ap_construct_url(r->pool, r->uri, r),
               ap_construct_url(r->pool, r->uri, r));
    for (; msg != NULL; msg = msg->next) {
        tms = localtime(&(msg->date));
        strftime(timestr, sizeof(timestr), "%Y/%m/%d %H:%M:%S", tms);
        timestr[sizeof(timestr) - 1] = '\0';

        body = ap_psprintf(r->pool,
                   "      <div class=\"mtb_entry\">\n"
                   "        <div class=\"mtb_title\"><a href=\"%s\" class=\"mtb_title\">%s</a></div>\n"
                   "        <div class=\"mtb_excerpt\">%s</div>\n"
                   "        <div class=\"mtb_blogname\">%s</div>\n"
                   "        <div class=\"mtb_date\">%s</div>\n"
                   "      </div>\n",
                   msg->url, msg->title,
                   msg->excerpt,
                   msg->blog_name,
                   timestr);
        body = (char *)mtb_convert_encoding(r->pool, cd, body);
        ap_rputs(body ? body : "", r);
    }
    ap_rputs("    </div>\n</body>\n</html>\n", r);
    mtb_iconv_close(cd);

    return OK;
}

mtb_module_rec mtb_contents_html_module = {
    NULL,
    NULL,
    NULL,
    send_html_contents
};
