/* Copyright (C) 2004 Hiroyuki OYAMA

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "module.h"
#include "trackback.h"
#include "encoding.h"
#include "util.h"


static int send_rss(request_rec *r, mtb_message_rec *msg)
{
    const char *encoding, *mode;
    char *body;
    table *param;
    struct tm *tms;
    unsigned char timestr[HUGE_STRING_LEN];
    iconv_t cd;

    param = (table *)ap_table_get(r->notes, REQUEST_PARAM_FIELD);
    mode = ap_table_get(param, "__mode");
    if (strcasecmp(mode, "rss") != 0)
        return DECLINED;

    encoding = ap_table_get(param, "charset");
    if (encoding == NULL) {
        encoding = "UTF-8";
    }
    encoding = mtb_find_encoding(encoding);
    if (encoding == NULL) {
        encoding = "UTF-8";
    }

    if ((cd = mtb_iconv_open(r->pool, "UTF-8", encoding)) == (iconv_t)(-1)) {
        /*
        ap_log_rerror(APLOG_MARK, APLOG_WARNING|APLOG_NOERRNO, r,
                      "cannot open iconv conversion descriptor: from=%s to=%s: %s",
                      "UTF-8", encoding, r->uri);
        */
    }


    r->content_type = "application/rdf+xml";
    ap_send_http_header(r);

    ap_rprintf(r,
               "<?xml version=\"1.0\" encoding=\"%s\"?>\n"
               "<response>\n"
               "  <error>0</error>\n"
               "  <rss version=\"0.91\">\n"
               "    <channel>\n"
               "      <title>TrackBack Server</title>\n"
               "      <link>%s</link>\n"
               "      <description>Generated by mod_trackback.c</description>\n",
               encoding,
               ap_construct_url(r->pool,
                                ap_pstrcat(r->pool, r->uri, "?",
                                           ap_escape_uri(r->pool, r->args),
                                           NULL),
                                r));
    for (; msg != NULL; msg = msg->next) {
        tms = localtime(&(msg->date));
        strftime(timestr, sizeof(timestr), "%Y/%m/%d %H:%M:%S", tms);
        timestr[sizeof(timestr) - 1] = '\0';

        body = ap_psprintf(r->pool,
                           "      <item>\n"
                           "        <title>%s</title>\n"
                           "        <link>%s</link>\n"
                           "        <description>%s</description>\n"
                           "      </item>\n",
                           msg->title, msg->url,
                           msg->excerpt);
        body = (char *)mtb_convert_encoding(r->pool, cd, body);
        ap_rputs(body ? body : "", r);
    }
    ap_rputs("    </channel>\n  </rss>\n</response>\n", r);
    mtb_iconv_close(cd);

    return OK;
}

mtb_module_rec mtb_contents_rss_module = {
    NULL,
    NULL,
    NULL,
    send_rss
};
